/******************************************************************************
* File Name: RTD.c
*  Version 2.1
*
* Description:
*  This file contains defines and function prototype for temperature measurement
*  using RTD sensor
*
* Note:
*
********************************************************************************
* Copyright (2012), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/
#include "RTD.h"

/* RTD scale factor*/
volatile float rtdScaleFactor = 1.0;

/* Flag for first sample */
uint8 firstRTDSample = TRUE;

/*******************************************************************************
* Function Name:  MeasureRTDTemp
********************************************************************************
* Summary:
* This function measures the temperature using two sub functions
* RTDGetResistance() and RTDRestoTemp()

* Parameters:  
* void:  
*
* Return: 
* float: Temperature in °C

* Side Effects:
* None 	
*******************************************************************************/
float MeasureRTDTemp(void)
{
    float rtdResistance;
	
	float rtdTemp;
	
	static float prevRTDTemp = 0.0;
	
	rtdResistance = RTDGetResistance();
	
	rtdTemp = RTDRestoTemp(rtdResistance);
	
	/* Reset the filter if previous reading and current reading is beyond 
     * threshold, this is done to make the filter response sharp */
    if((rtdTemp > (prevRTDTemp + FILTER_RESET)) || 
        (rtdTemp < (prevRTDTemp - FILTER_RESET)))
    {
         firstRTDSample = TRUE;
    }
    prevRTDTemp = rtdTemp;
       
    return rtdTemp;
}

/*******************************************************************************
* Function Name:  RTDGetResistance
********************************************************************************
* Summary:
* This function finds the RTD resistance.
* 
* Parameters:  
* void:  
*
* Return: 
* float: Resistance in ohms

* Theory: 
* This function finds the RTD resistance by measuring the voltage across 
* RTD sensor after applying current across RTD. Current given to RTD is calibrated 
* first by applying current to a known resistor and calculating the current back 
* by measuring the voltage across calibration resistor. This removes any error due
* to current which is supplied by the IDAC and actually flowing across RTD due to
* path resistance.  
*  
* To remove any offset error in the calibration resistor current path and RTD 
* current path, voltage is measured across both at zero current first. This zero
* current value is subtracted from both the resistor and sensor voltage reading
* obtained at 128 uA current.
* 
* Side Effects:
* None 	
*******************************************************************************/
float RTDGetResistance(void)
{

	/* Voltage read across calibration resistor and RTD at zero current to 
     * remove offset error */
    int32 zeroCurrentOffset = 0;
    
    /* Voltage read across calibration resistor at 1 mA current */
    int32 rtdCalResult = 0;
    
    /* Voltage read across RTD sensor at 1 mA current */
    int32 rtdResult = 0;
    
	/* RTD Resistance */
    float rtdResistance = 0.0;
    
    /* Filtered value for calibration voltage measurement */
    static float fRTDCalResult = 0.0;
    
    /* Filtered value for RTD sensor voltage measurement */
    static float fRTDResult = 0.0;
    
    /* Read zero current voltage across calibration resistor */
    zeroCurrentOffset = GetRcalOffset();
    
    /* Start IDAC set 2 mA range and set IDAC to source 1 mA current */
    IDAC_Start();

	IDAC_SetValue(128);
  
    /* Read voltage across calibration resistor */
    CurrentMux_FastSelect(CAL_IDAC_CHANNEL);
    ADCMux_FastSelect(CAL_CHANNEL);
    ADC_StartConvert();
    ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
    ADC_StopConvert();
    rtdCalResult = ADC_GetResult32();
    
    /* Get CDS difference */
    rtdCalResult = rtdCalResult - zeroCurrentOffset; 
    
    /* Filter signal */
    fRTDCalResult = FilterSignal(firstRTDSample, rtdCalResult, fRTDCalResult, RTD_FILTER_COEFF);
   
    /* Read voltage across RTD */
    CurrentMux_FastSelect(RTD_IDAC_CHANNEL);
    ADCMux_FastSelect(RTD_CHANNEL);
    ADC_StartConvert();
    ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
    ADC_StopConvert();
    rtdResult = ADC_GetResult32();
    IDAC_Stop();
    
    /* Read zero current voltage across RTD sensor resistor */
    zeroCurrentOffset = GetRTDOffset();
    
    /* Get CDS difference */
    rtdResult = rtdResult - zeroCurrentOffset;
    
    /* Filter signal */
    fRTDResult = FilterSignal(firstRTDSample, rtdResult, fRTDResult, RTD_FILTER_COEFF);
    
    firstRTDSample = FALSE;
    
	/* Calculate RTD resistance = Vrtd / Irtd = Vrtd / (Vref / Rref)*/
    rtdResistance = (fRTDResult * CAL_REF_RESISTOR)/fRTDCalResult;

	return rtdResistance;
}


/*******************************************************************************
* Function Name:  RTDRestoTemp
********************************************************************************
* Summary:
* This function converts RTD resistance to temperature
* 
* Parameters:  
* void:  
*
* Return: 
* float: Temperature in °C

* Theory: 
* RTD Resistance-Temperature relationship is defined by Callender Van Dusen 
* equations. RTD Temperature-Resistance relationship is obtained by the inverse 
* of Callender Van Dusen equations. It is not very easy to find the inverse 
* equation. Polynomial fit is done for the data and the temperature is obtained
* from the polynomial. A fifth order polynomial is used here to reduce error 
* <0.001 °C
* 
* Side Effects:
* None 	
*******************************************************************************/
float RTDRestoTemp(float rtdResistance)
{

	float rtdTemp;
	
	/* rtd resistance = rtd calibration constant * rtd measured resistance */
	rtdResistance = rtdScaleFactor * rtdResistance;
	
	/* If resistance > 100 use polynomial1
	 * rtdTemp = A0 * R^5 + A1 * R^4 + A2 * R^3 + A3 * R^2 + A4 * R^1 + A5 */
	if(rtdResistance > 100)
	{
		rtdTemp = ((((RTD_CALC_A0 * rtdResistance + RTD_CALC_A1) * rtdResistance + RTD_CALC_A2) * rtdResistance + 
					  RTD_CALC_A3) * rtdResistance + RTD_CALC_A4) * rtdResistance + RTD_CALC_A5; 						
	}
	/* If resistance < 100 use polynomial2 
	 *  rtdTemp = B0 * R^5 + B1 * R^4 + B2 * R^3 + B3 * R^2 +  B4 * R^1 +  B5 */
	else
	{
		rtdTemp = ((((RTD_CALC_B0 * rtdResistance + RTD_CALC_B1) * rtdResistance + RTD_CALC_B2) * rtdResistance + 
					  RTD_CALC_B3) * rtdResistance + RTD_CALC_B4) * rtdResistance + RTD_CALC_B5; 	
	}
	
	return rtdTemp;
	
}

/*******************************************************************************
*  Function Name: GetRTDOffset
********************************************************************************
* Summary:
* This function reads voltage across RTD sensor resistor at zero current which
* is used to remove offset error. 
*
* Parameters:
* void:
*
* Return:
* int32:  VSSA ref voltage for CDS
*
* Theory:
*
* Side Effects:
* None
*******************************************************************************/
int32 GetRTDOffset(void)
{
    int32 result = 0;
       
    IDAC_Start();

	IDAC_SetValue(0);

    CurrentMux_FastSelect(RTD_IDAC_CHANNEL);
    ADCMux_FastSelect(RTD_CHANNEL);

    ADC_StartConvert();
    ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
    result = ADC_GetResult32();
    ADC_StopConvert();
    
    IDAC_Stop();
    
    return result;
}


/* [] END OF FILE */

