/******************************************************************************
* File Name: diode.c
*  Version 2.1
*
* Description:
*  This file contains code for temperature measurement using Diode sensor
*
* Note:
*
********************************************************************************
* Copyright (2012), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/
#include "diode.h"

/* Diode temp scale factor for calibration */
volatile float diode1ScaleFactor = 1.0 , diode2ScaleFactor = 1.0;


/*******************************************************************************
* Function Name:  MeasureFirstDiodeTemp
********************************************************************************
* Summary:
* This function measures the temperature using diode1 by measuring the voltage
* across the base-emitter junction of the diode at two different currentS.
* First, current of 10µA is given to diode and voltage drop across it's 
* Base-Emitter junction is measured. Next, 100µA is given to diode and again
* voltage drop across Base-Emitter junction is measured. The difference in the
* voltage drop across the diode at two given current is proportional to the
* temperature.
*  
* Parameters:  
* void:  
*
* Return: 
* float: Temperature in °C

* Theory:
* Following equation is used to measure the temperature:
* 
* T =  q/(η*k)*((V2-V1)/ln(I2/I1))
* where
* T = temperature in Kelvin
* q = magnitude of electronic charge(1.602 * 10e-19)
* k = Boltzmanns constant(1.38 * 10e-23)
* η = Ideality factor for diode, typically this value varies from 1.001 to 1.01 
* V2 = voltage drop(Vbe) at I2
* V1 = voltage drop(Vbe) at I1
* 
* Side Effects:
* None 	
*******************************************************************************/
float MeasureFirstDiodeTemp(void)
{
    /* Variable to store zero current offset across calibration resistor */
    int32 zeroCurrentOffset = 0;
    
    /* Diode sensor temperature */
    float diodeTemp = 0.0;
    
    /* ADC voltage read across calibration resistor */
    int32 diodeCalResult = 0;
    
    /* ADC voltage read across base emitter junction of diode */
    int32 diodeResult = 0;
    
    /* Filtered value for voltage drop across calibration resistor 
    *  at 10uA current */
    static float fDiodeCalResult_1 = 0.0;
    
    /* Filtered value for diode Vbe measurement at 10uA current */
    static float fDiodeResult_1 = 0.0;
    
    /* Filtered value for voltage drop across calibration resistor 
    *  at 100uAmp current */
    static float fDiodeCalResult_2 = 0.0;
    
    /* Filtered value for diode Vbe measurement at 100 uA */
    static float fDiodeResult_2 = 0.0;
    
    /* Flag for first sample */
    static uint8 firstDiodeSample = TRUE;
    
    /* Voltage difference of Vbe at 10 and 100 uA */
    float vDiff = 0.0;
    int32 countDiff = 0;
    
    /* Log value of current */
    float currentLogR = 0.0;

    uint8 cntr;
    
    static float prevDiodeTemp = 0.0;
	    
    /* Read zero current voltage across calibration resistor */
    zeroCurrentOffset = GetRcalOffset();
	
	/* Start IDAC to source current */
    IDAC_Start();
	IDAC_SetValue(0);
    
    /* First time Vbe is measured with IDAC value set to 10uA,
     * second time Vbe is measured with IDAC value set to 100uA */
    for(cntr = 0; cntr <= 1; cntr++)
    {
        /* First time IDAC value set to 10uA,
         * second time it is set to 100uA */
        if(cntr == 0)
        {
            /* Set IDAC for 10uA current */
            IDAC_SetValue(10);
        }
        else
        {
            /* Set IDAC for 100uA current */
            IDAC_SetValue(100);
        }
    
		/* Direct current through calibration resistor */
        CurrentMux_FastSelect(CAL_IDAC_CHANNEL);
		
		/* Select calibration channel as amux output and measure voltage across 
		 * calibration resistor  using ADC */
        ADCMux_FastSelect(CAL_CHANNEL);
        ADC_StartConvert();
        ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
        diodeCalResult = ADC_GetResult32();
		
		/* Stop ADC conversion */
        ADC_StopConvert();
		
        /* Get CDS difference */
        diodeCalResult = diodeCalResult - zeroCurrentOffset; 
        
        /* Filter and Store calibration result at 10uA */
        if(cntr == 0)
        {
            fDiodeCalResult_1  = FilterSignal(firstDiodeSample, diodeCalResult, 
                                     fDiodeCalResult_1, DIODE_CAL_FILTER_COEFF);
        }
        
        /* Filter and Store calibration result at 100uA */
        else
        {
            fDiodeCalResult_2  = FilterSignal(firstDiodeSample, diodeCalResult, 
                                     fDiodeCalResult_2, DIODE_CAL_FILTER_COEFF);
        }
        
        /* Connect current Mux to p3_6 and connect p3_7 to ground */
        CurrentMux_FastSelect(DIODE_1_IDAC_CHANNEL);
		Diode_3_6_Write(1);
		Diode_3_7_Write(0);
		
		/* Select Diode as amux output and measure voltage 
		 * across base emitter junction(diode) using ADC */
        ADCMux_FastSelect(DIODE_CHANNEL);
        ADC_StartConvert();
        ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
        ADC_StopConvert();
        diodeResult = ADC_GetResult32();
        
        /* Filter and Store Vbe at 10uA */
        if(cntr == 0)
        {
            fDiodeResult_1  = FilterSignal(firstDiodeSample, diodeResult, 
                                           fDiodeResult_1, DIODE_FILTER_COEFF);
        } 
        
        /* Filter and Store Vbe at 100uA */
        else
        {
            fDiodeResult_2  = FilterSignal(firstDiodeSample, diodeResult, 
                                           fDiodeResult_2, DIODE_FILTER_COEFF);
            firstDiodeSample = FALSE;
        }
    }
    IDAC_Stop();
	
	Diode_3_6_Write(1);
	Diode_3_7_Write(1);

    /* Calculate (V2-V1)*/ 
	countDiff = fDiodeResult_2 - fDiodeResult_1;
    vDiff = ADC_CountsTo_Volts(countDiff);
	
	/* Calculate ln(I2/I1) = ln ((I2 * R)/(I1 * R))*/
    currentLogR = log(fDiodeCalResult_2 / fDiodeCalResult_1);
	
	/* T =  scale factor * [q/(η*k)] * ((V2-V1) / ln(I2/I1)) */
    diodeTemp = diode1ScaleFactor * DIODE_CONSTANT  * (vDiff / currentLogR);
	
	/* Subtract ZERO_DEGREE_CELSIUS_KELVIN from T to convert the temp to °C */
    diodeTemp -= ZERO_DEGREE_CELSIUS_KELVIN;
	
	/* Reset the filter if previous reading and current reading is beyond 
     * threshold, this is done to make the filter response sharp */
    if((diodeTemp > (prevDiodeTemp + FILTER_RESET)) || 
        (diodeTemp < (prevDiodeTemp - FILTER_RESET)))
    {
         firstDiodeSample = TRUE;
    }
    prevDiodeTemp = diodeTemp;
    
    return diodeTemp;
}   

/*******************************************************************************
* Function Name:  MeasureSecondDiodeTemp
********************************************************************************
* Summary:
* This function measures the temperature using diode2 by measuring the voltage
* across the base-emitter junction of the diode at two different currentS.
* First, current of 10µA is given to diode and voltage drop across it's 
* Base-Emitter junction is measured. Next, 100µA is given to diode and again
* voltage drop across Base-Emitter junction is measured. The difference in the
* voltage drop across the diode at two given current is proportional to the
* temperature.
*  
* Parameters:  
* void:  
*
* Return: 
* float: Temperature in °C

* Theory:
* Following equation is used to measure the temperature:
* 
* T =  q/(η*k)*((V2-V1)/ln(I2/I1))
* where
* T = temperature in Kelvin
* q = magnitude of electronic charge(1.602 * 10e-19)
* k = Boltzmanns constant(1.38 * 10e-23)
* η = Ideality factor for diode, typically this value varies from 1.001 to 1.01 
* V2 = voltage drop(Vbe) at I2
* V1 = voltage drop(Vbe) at I1
* 
* Side Effects:
* None 
*******************************************************************************/
float MeasureSecondDiodeTemp(void)
{
    /* Voltage read across calibration resistor and RTD at zero current to 
     * remove offset error */
    int32 zeroCurrentOffset = 0;
    
    /* Diode sensor temperature */
    float diodeTemp = 0.0;
    
    /* ADC voltage read across calibration resistor */
    int32 diodeCalResult = 0;
    
    /* ADC voltage read across base emitter junction of diode */
    int32 diodeResult = 0;
    
    /* Filtered value for voltage drop across calibration resistor 
    *  at 10uAmp current */
    static float fDiodeCalResult_1 = 0.0;
    
    /* Filtered value for diode Vbe measurement at 10uAmp current */
    static float fDiodeResult_1 = 0.0;
    
    /* Filtered value for voltage drop across calibration resistor 
    *  at 100uAmp current */
    static float fDiodeCalResult_2 = 0.0;
    
    /* Filtered value for diode Vbe measurement at 100 microAmp */
    static float fDiodeResult_2 = 0.0;
    
    /* Flag for first sample */
    static uint8 firstDiodeSample = TRUE;
    
    /* Voltage difference of Vbe at 10 and 100 Micro Amp */
    float vDiff = 0.0;
	
    /* ADC count difference corresponding to vDiff */
	int32 countDiff = 0;
	
	/* Previous diode temperature stored in a variable for filter */
    static float prevDiodeTemp = 0.0;
    
    /* Log value of current */
    float currentLogR = 0.0;

    uint8 i = 0;
	
	

    /* Read zero current voltage across calibration resistor */
    zeroCurrentOffset = GetRcalOffset();
	
	/* Start IDAC to source current */
    IDAC_Start();
	IDAC_SetValue(0);
    
    for(i = 0; i <= 1; i++)
    {
        /* First time Vbe measurement with IDAC value set to 10 uA,
           second time it is set to 100 uA */
        if(i == 0)
        {
            /* Set IDAC for 10 uA current */
            IDAC_SetValue(10);
        }
        else
        {
            /* Set IDAC for 100 uA current */
            IDAC_SetValue(100);
        }
           
		/* Direct current through calibration resistor */
        CurrentMux_FastSelect(CAL_IDAC_CHANNEL);
		
		/* Select calibration channel as amux output and measure voltage across 
		 * calibration resistor  using ADC */
        ADCMux_FastSelect(CAL_CHANNEL);
        ADC_StartConvert();
        ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
        diodeCalResult = ADC_GetResult32();
		
		/* Stop ADC conversion */
        ADC_StopConvert();
        
        /* Get CDS difference */
        diodeCalResult = diodeCalResult - zeroCurrentOffset; 
        
        /* Filter and Store calibration result at 10uA */
        if(i == 0)
        {
            fDiodeCalResult_1  = FilterSignal(firstDiodeSample, diodeCalResult,  
                                     fDiodeCalResult_1, DIODE_CAL_FILTER_COEFF);
        }
       
        /* Filter and Store calibration result at 100uA */
        else
        {
            fDiodeCalResult_2  = FilterSignal(firstDiodeSample, diodeCalResult,  
                                     fDiodeCalResult_2, DIODE_CAL_FILTER_COEFF);
        }
        
        /* Connect IDAC to p3_7 and connect p3_6 to ground */
        CurrentMux_FastSelect(DIODE_2_IDAC_CHANNEL);
        Diode_3_6_Write(0);
		Diode_3_7_Write(1);
		
		/* Select diode channel as amux output 
		 * and measure the voltage across diode2 */
		ADCMux_FastSelect(DIODE_CHANNEL);
        ADC_StartConvert();
        ADC_IsEndConversion(ADC_WAIT_FOR_RESULT);
        ADC_StopConvert();
        diodeResult = ADC_GetResult32();
        
        /* Filter and Store Vbe at 10uA */
        if(i == 0)
        {
            fDiodeResult_1  = FilterSignal(firstDiodeSample, diodeResult,
                                           fDiodeResult_1, DIODE_FILTER_COEFF);
        } 
        /* Filter and Store Vbe at 100uA */
        else
        {
            /* IIR filter */
            fDiodeResult_2  = FilterSignal(firstDiodeSample, diodeResult, 
                                           fDiodeResult_2, DIODE_FILTER_COEFF);
            firstDiodeSample = FALSE;
        }
    }
    IDAC_Stop();
	
	/* Remove p3_6 and p3_7 from ground */
	Diode_3_6_Write(1);
	Diode_3_7_Write(1);

    /* Calculate (V2 - V1)*/ 
	countDiff = fDiodeResult_1 - fDiodeResult_2;
    vDiff = ADC_CountsTo_Volts(countDiff);
	
	/* Calculate log(I2 / I1) = log ((I2 * R)/(I1 * R))*/
    currentLogR = log(fDiodeCalResult_2 / fDiodeCalResult_1);
	
	/* T =  scale factor * [q/(η*k)] * ((V2-V1) / ln(I2/I1)) */
    diodeTemp = diode2ScaleFactor * DIODE_CONSTANT  * (vDiff / currentLogR);
	
	/* Subtract ZERO_DEGREE_CELSIUS_KELVIN from T to convert the temp to °C */
    diodeTemp -= ZERO_DEGREE_CELSIUS_KELVIN;

    /* Reset the filter if previous reading and current reading is beyond 
     * threshold, this is done to make the filter response sharp */
    if((diodeTemp > (prevDiodeTemp + FILTER_RESET)) || 
        (diodeTemp < (prevDiodeTemp - FILTER_RESET)))
    {
         firstDiodeSample = TRUE;
    }
	
    prevDiodeTemp = diodeTemp;
    
    return diodeTemp;  

}




/* [] END OF FILE */

