/******************************************************************************
* File Name: filter.c
*  Version 2.1
*
* Description:
*  This file contains code to filter sensor output
*
* Note:
*
********************************************************************************
* Copyright (2012), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/
#include "filter.h"

/*******************************************************************************
*  Function Name: FilterSignal
********************************************************************************
* Summary:
*  This is IIR filter to process the sampled ADC data. For first sample passed 
*  the function simply returns the passed ADC sample. For second sample onwards,
*  fraction of input signal based upon the filter coefficient, is calculated and 
*  added to the previously stored filtered value.
*
* Parameters:
* (uint8)  firstSampleFlag - indicated if the sample passed is first sample or 
                             not, this is also used to reset the filter
*
* (int32)  ADCSample       - ADC sample read for given signal
* (float)  filteredValue   - stored filtered value for given sensor
* (uint16) filterCoeff     - fraction of input signal which should be filtered
*
* Return: 
* (float)filteredValue - filtered value passed is updated after applying 
* IIR filter to the ADC sample.
*
* Theory:
* For more information on the IIR filter please refer to Cypress application
* note AN2099.
*
* Side Effects:
* None
*******************************************************************************/
float FilterSignal(uint8 firstSampleFlag, int32 ADCSample, float filteredValue, uint16 filterCoeff)
{
    float temp = 0.0;
    
    /* If first sample is passed then just store this value as filtered value */
    if(firstSampleFlag)
    {
        filteredValue = (float)ADCSample;
    }
    
    /* If not the first sample then based on filter coefficient, filter the 
     * input signal */
    else
    {
        temp = ((float)ADCSample - filteredValue) / filterCoeff;
        filteredValue = filteredValue + temp;
    }
    return filteredValue;
}  

/* [] END OF FILE */
